USE [master];
-- change to your repository for the sp_WOXCompliant_CJB procedure and outputs, we use our copy of master for testing.
-- We recommend setting up a separate database for the DBA to use when monitoring.
GO
SET NOCOUNT ON;
-- Check to see if the sp_WOXCompliant_CJB procedure is in place already, if not create it
IF OBJECT_ID('dbo.sp_WOXCompliant_Auto') IS NULL
    EXEC ( 'CREATE PROCEDURE dbo.sp_WOXCompliant_Auto AS RETURN 0;' );
GO

-- Updating the procedure
ALTER PROCEDURE [dbo].[sp_WOXCompliant_Auto]
    @CheckType TINYINT = 0
  , @Help TINYINT = 0
  , @OutputType VARCHAR(20) = 'TABLE'
  , @EnablePS BIT = 1
  , @LeaveCmd BIT = 0
  , @Version INT = NULL OUTPUT
  , @VersionDate DATETIME = NULL OUTPUT
AS
    SET NOCOUNT ON;
    SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;

    /* Check is xp_cmdshell is active and leave that way to prevent breaking existing systems */
    SELECT @LeaveCmd = ISNULL(COUNT(1), 0)
    FROM   sys.configurations
    WHERE  name = 'xp_cmdshell'
           AND value = 1;

    /* Ensure xp_cmdshell is on for running the processes below.*/
    EXEC sp_configure 'show advanced options'
                    , 1;
    RECONFIGURE;
    EXEC sp_configure 'xp_cmdshell'
                    , 1;
    RECONFIGURE;

    SELECT @Version = 004
         , @VersionDate = '20171207';

    IF @Help = 1
        PRINT '
	/*
	sp_WOXCompliant_Auto (TM) v004 - December 2017

	(C) 2017, WaterOx Consulting, Inc.
	See https://WaterOxConsulting.com/eula for the End User Licensing Agreement.

	To learn more, visit https://www.WaterOxConsulting.com/Compliance where you can download
	new versions for free, learn to use the script, get more info on
	the findings, and more.  To contribute code and see your name in the change
	log, email your improvements & checks to Compliance@WaterOxConsulting.com


	Known limitations of this version:
	 - Performs basic checks and presents reminders for US DoD STIG complaince based on stigs updated in 2017 for SQL Server 2012 Database and Database Instance
	 - Updated to use Powershell 5 and higher. Lower versions installed on the server may not return all information.
	 - Earlier STIG requirements may be added, if required/requested.
	 - No support for SQL Server 2000 or compatibility mode 80.
	 - If a database name has a question mark in it, some tests may fail.
	 - System databases are not checked for compliance.
	 - This script has not been tested against Azure VMs or SQL Azure. We welcome feedback from anyone who has tried this.
	 - Work is still underway for the reference links in the results from the script.

    Not all compliance requirements for SQL Server can be checked by a script, but we can look for indicators to raise some alerts.
    Results are stored in a temp table to then be dumped out to the screen at the end. Future releases may add the option to export to a 
	specified table in an audit database.

	Parameter explanations:

	@CheckType					1 = DoD STIGs (More options, such as PCI, HIPAA, etc. to be added in future releases)
	@Help						1 = Show imbedded help information
	@EnablePS BIT				1 = Perform the checks that require PowerShell
    @LeaveCmd BIT				1 = Leave SQLCMD enabled after script completed, default is 0 to disable.

	Sample syntax: 
	dbo.sp_WOXCompliant_CJB @checktype = 1

	Change Log: 
		12/7/2017
			- Script modified to be part of new SP_WOXAudit script which is easily customized to include other PS and SQL Scripts and generates an output file with all data for each server checked.
			- Major overhaul to update to include checks for all DOD Stigs required for Public III - Administrative Public STIGs dated 2017-04-03
			- A good number of instance level checks can not be automated. They are referenced with notes on what has to be done to validate the requirements

		09/30/2015
			- Fixed error when running against SQL Server with case sensitive collation
			- Added check & message for orphaned users when checking V-41395
			- Added checks for the following STIGS
				V-40909		V-41258		V-40935		V-41285		V-41299
				V-40946		V-41259		V-41278		V-41284		V-40938
				V-55805		V-41302		V-41279		V-41044		V-40939
				V-41247		V-41303		V-41276		V-41289		V-40943
				V-41246		V-41300		V-41277		V-41288		V-40942
				V-41249		V-41206		V-41274		V-40921		
				V-59915		V-41261		V-41275		V-41209
				V-41254		V-41260		V-41272		V-41208
				V-41255		V-41263		V-41273		V-41207
				V-41256		V-41262		V-41270		V-41292
				V-41257		V-41265		V-41271		V-41293
				V-41250		V-41264		V-41283		V-41294
				V-41251		V-41267		V-41281		V-41295
				V-41252		V-41266		V-41280		V-41296
				V-41253		V-41268		V-41287		V-41297
				V-40940		V-41248		V-41286		V-41298
				*/';
    ELSE
        BEGIN
            /* Declare variables used through rest of the check script*/

            DECLARE @ProductVersion NVARCHAR(128)
                  , @ProductVersionMajor DECIMAL(10, 2)
                  , @ProductVersionMinor DECIMAL(10, 2)
                  , @cdt INT
                  , @ChkInstanceName NVARCHAR(128)
                  , @ChkSrvName NVARCHAR(128)
                  , @TrueSrvName NVARCHAR(128)
                  , @SQLSrv NVARCHAR(128)
                  , @PhysicalSrvName NVARCHAR(128)
                  , @REGKEY NVARCHAR(128)
                  , @TCount SMALLINT
                  , @TRACEID INT
                  , @AuditLevel INT
                  , @ErrorLogs INT
                  , @CBS NVARCHAR(150)
                  , @InstanceName VARCHAR(100)
                  , @InstanceLocation VARCHAR(100)
                  , @InstancePath VARCHAR(100);;


            /* Build the table to store results */
            IF OBJECT_ID('tempdb..#WOXResults') IS NOT NULL
                DROP TABLE #WOXResults;
            CREATE TABLE #WOXResults
                (
                    ID INT IDENTITY(1, 1)
                  , CheckID INT
                  , DatabaseName NVARCHAR(128)
                  , Severity TINYINT
                  , CheckGroup VARCHAR(50)
                  , Finding VARCHAR(200)
                  , URL VARCHAR(200)
                  , Details NVARCHAR(4000)
                );


            /* this is where other check temp tables can go */

            CREATE TABLE #net_data
                (
                    descriptions VARCHAR(256)
                );
            CREATE TABLE #FoldPerm
                (
                    GNAME VARCHAR(125)
                  , UName VARCHAR(125)
                  , AccessType VARCHAR(255)
                  , Rights VARCHAR(255)
                );

            CREATE TABLE #cbs
                (
                    CBS NVARCHAR(150)
                );

            CREATE TABLE #Versions
                (
                    Ver VARCHAR(15)
                );

            CREATE TABLE #DBUs
                (
                    DBUID INT IDENTITY(1, 1)
                  , DBUserName VARCHAR(125)
                );

            CREATE TABLE #CSES
                (
                    Name VARCHAR(50)
                  , Minimum INT
                  , Maximum INT
                  , Config_value INT
                  , Run_value INT
                );

            CREATE TABLE #RegResult
                (
                    ResultValue NVARCHAR(4)
                );

            CREATE TABLE #Svcs
                (
                    SVCID INT IDENTITY(1, 1)
                  , Svc VARCHAR(128)
                  , SvcName VARCHAR(128)
                );

            CREATE TABLE #SQLTraces
                (
                    TID INT IDENTITY(1, 1)
                  , TraceID SMALLINT
                );

            CREATE TABLE #ReqTraces
                (
                    EventID SMALLINT
                );

            CREATE TABLE #events
                (
                    EventID SMALLINT
                );
            CREATE TABLE #AdminPerms
                (
                    Perm VARCHAR(125)
                );

            /* Prepopulate any temp tables*/
            BEGIN
                INSERT #AdminPerms ( Perm )
                VALUES ( 'fn_yukonsecuritymodelrequired' )
                     , ( 'sp_add_agent_parameter' )
                     , ( 'sp_add_agent_profile' )
                     , ( 'sp_adddatatype' )
                     , ( 'sp_adddistributiondb' )
                     , ( 'sp_adddistributor' )
                     , ( 'sp_addqreader_agent' )
                     , ( 'sp_addsubscriber' )
                     , ( 'sp_addsubscriber_schedule' )
                     , ( 'sp_addtabletocontents' )
                     , ( 'sp_attachsubscription' )
                     , ( 'sp_cdc_cleanup_change_table' )
                     , ( 'sp_cdc_disable_db' )
                     , ( 'sp_cdc_disable_table' )
                     , ( 'sp_cdc_drop_job' )
                     , ( 'sp_cdc_enable_db' )
                     , ( 'sp_cdc_enable_table' )
                     , ( 'sp_cdc_restoredb' )
                     , ( 'sp_cdc_vupgrade' )
                     , ( 'sp_certify_removable' )
                     , ( 'sp_change_agent_parameter' )
                     , ( 'sp_change_agent_profile' )
                     , ( 'sp_change_subscription_properties' )
                     , ( 'sp_change_users_login' )
                     , ( 'sp_changedistpublisher' )
                     , ( 'sp_changedistributiondb' )
                     , ( 'sp_changedistributor_password' )
                     , ( 'sp_changedistributor_property' )
                     , ( 'sp_changemergesubscription' )
                     , ( 'sp_changeqreader_agent' )
                     , ( 'sp_changereplicationserverpasswords' )
                     , ( 'sp_changesubscriptiondtsinfo' )
                     , ( 'sp_checkinvalidivarticle' )
                     , ( 'sp_copysubscription' )
                     , ( 'sp_create_removable' )
                     , ( 'sp_cycle_errorlog' )
                     , ( 'sp_dbcmptlevel' )
                     , ( 'sp_dbmmonitoraddmonitoring' )
                     , ( 'sp_dbmmonitorchangealert' )
                     , ( 'sp_dbmmonitordropalert' )
                     , ( 'sp_dbmmonitordropmonitoring' )
                     , ( 'sp_dbmmonitorhelpalert' )
                     , ( 'sp_dbmmonitorhelpmonitoring' )
                     , ( 'sp_dbmmonitorresults' )
                     , ( 'sp_dbmmonitorupdate' )
                     , ( 'sp_dbremove' )
                     , ( 'sp_drop_agent_parameter' )
                     , ( 'sp_drop_agent_profile' )
                     , ( 'sp_dropdatatypemapping' )
                     , ( 'sp_dropdistpublisher' )
                     , ( 'sp_dropdistributiondb' )
                     , ( 'sp_dropdistributor' )
                     , ( 'sp_dropmergepullsubscription' )
                     , ( 'sp_droppullsubscription' )
                     , ( 'sp_dropsubscriber' )
                     , ( 'sp_dsninfo' )
                     , ( 'sp_enumdsn' )
                     , ( 'sp_flush_commit_table_on_demand' )
                     , ( 'sp_generate_agent_parameter' )
                     , ( 'sp_get_distributor' )
                     , ( 'sp_get_Oracle_publisher_metadata' )
                     , ( 'sp_getagentparameterlist' )
                     , ( 'sp_getdefaultdatatypemapping' )
                     , ( 'sp_grant_publication_access' )
                     , ( 'sp_help_agent_default' )
                     , ( 'sp_help_agent_parameter' )
                     , ( 'sp_help_agent_profile' )
                     , ( 'sp_helpdistpublisher' )
                     , ( 'sp_helpdistributor' )
                     , ( 'sp_helpmergesubscription' )
                     , ( 'sp_helpqreader_agent' )
                     , ( 'sp_helpreplicationdboption' )
                     , ( 'sp_identitycolumnforreplication' )
                     , ( 'sp_IHValidateRowFilter' )
                     , ( 'sp_IHXactSetJob' )
                     , ( 'sp_link_publication' )
                     , ( 'sp_monitor' )
                     , ( 'sp_MSadd_distribution_agent' )
                     , ( 'sp_MSadd_logreader_agent' )
                     , ( 'sp_MSadd_merge_agent' )
                     , ( 'sp_MSadd_snapshot_agent' )
                     , ( 'sp_MSadd_subscriber_schedule' )
                     , ( 'sp_MSadd_tracer_history' )
                     , ( 'sp_MSadd_tracer_token' )
                     , ( 'sp_MScdc_cleanup_job' )
                     , ( 'sp_MScdc_db_ddl_event' )
                     , ( 'sp_MScdc_ddl_event' )
                     , ( 'sp_MSchange_distribution_agent_properties' )
                     , ( 'sp_MSchange_logreader_agent_properties' )
                     , ( 'sp_MSchange_merge_agent_properties' )
                     , ( 'sp_MSchange_snapshot_agent_properties' )
                     , ( 'sp_MSchangedynamicsnapshotjobatdistributor' )
                     , ( 'sp_MSchangedynsnaplocationatdistributor' )
                     , ( 'sp_MScheck_pull_access' )
                     , ( 'sp_MScleanupmergepublisher_internal' )
                     , ( 'sp_MSclear_dynamic_snapshot_location' )
                     , ( 'sp_MScreate_dist_tables' )
                     , ( 'sp_MSdbuserpriv' )
                     , ( 'sp_MSdeletefoldercontents' )
                     , ( 'sp_MSdrop_6x_replication_agent' )
                     , ( 'sp_MSdrop_merge_agent' )
                     , ( 'sp_MSdrop_snapshot_dirs' )
                     , ( 'sp_MSdropmergedynamicsnapshotjob' )
                     , ( 'sp_MSdynamicsnapshotjobexistsatdistributor' )
                     , ( 'sp_MSenumallpublications' )
                     , ( 'sp_MSfetchAdjustidentityrange' )
                     , ( 'sp_MSfix_6x_tasks' )
                     , ( 'sp_MSforce_drop_distribution_jobs' )
                     , ( 'sp_MSget_agent_names' )
                     , ( 'sp_MSget_jobstate' )
                     , ( 'sp_MSget_oledbinfo' )
                     , ( 'sp_MSget_publication_from_taskname' )
                     , ( 'sp_MSgetdbversion' )
                     , ( 'sp_MSgetmaxsnapshottimestamp' )
                     , ( 'sp_MShelp_repl_agent' )
                     , ( 'sp_MShelp_replication_status' )
                     , ( 'sp_MShelp_snapshot_agent' )
                     , ( 'sp_MShelpconflictpublications' )
                     , ( 'sp_MShelpdynamicsnapshotjobatdistributor' )
                     , ( 'sp_MShelplogreader_agent' )
                     , ( 'sp_MShelpsnapshot_agent' )
                     , ( 'sp_MShelptranconflictcounts' )
                     , ( 'sp_MSinit_publication_access' )
                     , ( 'sp_MSreinit_failed_subscriptions' )
                     , ( 'sp_MSremoveoffloadparameter' )
                     , ( 'sp_MSrepl_backup_complete' )
                     , ( 'sp_MSrepl_backup_start' )
                     , ( 'sp_MSrepl_createdatatypemappings' )
                     , ( 'sp_MSrepl_dropdatatypemappings' )
                     , ( 'sp_MSrepl_enumarticlecolumninfo' )
                     , ( 'sp_MSrepl_enumpublications' )
                     , ( 'sp_MSrepl_enumpublishertables' )
                     , ( 'sp_MSrepl_enumsubscriptions' )
                     , ( 'sp_MSrepl_enumtablecolumninfo' )
                     , ( 'sp_MSrepl_getdistributorinfo' )
                     , ( 'sp_MSrepl_startup_internal' )
                     , ( 'sp_MSreplagentjobexists' )
                     , ( 'sp_MSreplcheck_permission' )
                     , ( 'sp_MSreplcheck_pull' )
                     , ( 'sp_MSreplcheck_subscribe' )
                     , ( 'sp_MSreplcheck_subscribe_withddladmin' )
                     , ( 'sp_MSreplcopyscriptfile' )
                     , ( 'sp_MSreplremoveuncdir' )
                     , ( 'sp_MSsetalertinfo' )
                     , ( 'sp_MSSetServerProperties' )
                     , ( 'sp_MSsetupnosyncsubwithlsnatdist' )
                     , ( 'sp_MSsetupnosyncsubwithlsnatdist_cleanup' )
                     , ( 'sp_MSsetupnosyncsubwithlsnatdist_helper' )
                     , ( 'sp_MSstartdistribution_agent' )
                     , ( 'sp_MSstartmerge_agent' )
                     , ( 'sp_MSstartsnapshot_agent' )
                     , ( 'sp_MSstopdistribution_agent' )
                     , ( 'sp_MSstopmerge_agent' )
                     , ( 'sp_MSstopsnapshot_agent' )
                     , ( 'sp_MSupdate_agenttype_default' )
                     , ( 'sp_oledbinfo' )
                     , ( 'sp_procoption' )
                     , ( 'sp_removedbreplication' )
                     , ( 'sp_removesrvreplication' )
                     , ( 'sp_replication_agent_checkup' )
                     , ( 'sp_replicationdboption' )
                     , ( 'sp_resetstatus' )
                     , ( 'sp_restoredbreplication' )
                     , ( 'sp_SetAutoSAPasswordAndDisable' )
                     , ( 'sp_setdefaultdatatypemapping' )
                     , ( 'sp_updatestats' )
                     , ( 'sp_validatelogins' )
                     , ( 'sp_vupgrade_mergeobjects' )
                     , ( 'sp_vupgrade_replication' )
                     , ( 'sp_vupgrade_replsecurity_metadata' )
                     , ( 'xp_repl_convert_encrypt_sysadmin_wrapper' );
            END;

            /* get SQL version and name info*/

            SET @PhysicalSrvName = CAST(SERVERPROPERTY('MachineName') AS VARCHAR(128));
            SET @ChkSrvName = CAST(SERVERPROPERTY('INSTANCENAME') AS VARCHAR(128));
            SET @ChkInstanceName = @@serverName;
            SET @ProductVersion = CAST(SERVERPROPERTY('ProductVersion') AS NVARCHAR(128));
            SELECT @ProductVersionMajor = SUBSTRING(
                                              @ProductVersion
                                            , 1
                                            , CHARINDEX('.', @ProductVersion)
                                              + 1)
                 , @ProductVersionMinor = PARSENAME(
                                              CONVERT(
                                                  VARCHAR(32)
                                                , @ProductVersion)
                                            , 2);

            /* Set other high level variables*/

            BEGIN TRY
                EXEC master.dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE'
                                                  , N'Software\Microsoft\MSSQLServer\MSSQLServer'
                                                  , N'AuditLevel'
                                                  , @AuditLevel OUTPUT;
            END TRY
            BEGIN CATCH
                SET @AuditLevel = 0; -- No auditing
            END CATCH;

            BEGIN TRY
                EXEC master.dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE'
                                                  , N'Software\Microsoft\MSSQLServer\MSSQLSERVER'
                                                  , N'NumErrorLogs'
                                                  , @ErrorLogs OUTPUT;
            END TRY
            BEGIN CATCH
                SET @ErrorLogs = 6; -- default minimum value
            END CATCH;

            /* Check if powershell is on the system, if not change flag to prevent powershell checks */
            IF @EnablePS = 1
                BEGIN
                    EXECUTE master..xp_regread 'HKEY_LOCAL_MACHINE'
                                             , 'SOFTWARE\Microsoft\PowerShell\3' -- 3 is for PS 4 and higher - need to do anothe check for the actual version number being >= 5
                                             , 'Install'
                                             , @EnablePS OUTPUT;
                END;
            SET @EnablePS = ISNULL(@EnablePS, 0);

            IF ISNULL(@EnablePS, 0) = 0
                INSERT #WOXResults ( CheckID
                                   , DatabaseName
                                   , Severity
                                   , CheckGroup
                                   , Finding
                                   , URL
                                   , Details )
                VALUES ( 0, N'' -- DatabaseName - nvarchar(128)
                       , 1, 'CONFIG', 'sp_WOXCompliant_CJB - PowerShell', ''
                       , 'PowerShell is not installed, not available, or the EnablePS Flag was set to 0. Checks that use PowerShell will not be processed.' );


            /* The checks start here 
This is the meat of the script. These are the checks that are done for each DB STIG in this initial version.
*/

            /* Check type of check being done*/
            /*1 - V41398 - PublicAccounts*/
            IF @CheckType IN ( 1 )
                BEGIN
                    EXEC sp_MSforeachdb 'use [?];					
					IF DB_ID(''?'') > 4
					INSERT  INTO #WOXResults
						( CheckID ,
						  Databasename ,
						  Severity ,
						  CheckGroup ,
						  Finding ,
						  URL ,
						  Details
						)
						SELECT  1 AS CheckID ,
							DB_Name() ,
							1 AS Priority ,
							''DATABASE'' AS FindingGroup ,
							''V-41398''  AS Finding ,
							''http://WaterOxConsulting.com/archives/Compliance/PublicAccounts'' AS URL ,
							''A possibly public account named [''+sp.name+''] was found in ['' + DB_NAME() + ''] and should be checked and removed if public.''
						FROM sys.sysusers su
						LEFT JOIN sys.server_principals sp
						ON su.sid = sp.sid
						WHERE ( su.name like ''publ%''
						OR sp.name like ''publ%'' )
						AND NOT su.sid = CONVERT(VARBINARY(85), 0x)
						and sp.name is not null';

                END;


            /*2 - V40907 - SecureConnections*/
            IF @CheckType IN ( 1 )
                BEGIN

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 2 AS CheckID
                                     , @@SERVERNAME
                                     , 1 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40907' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/SecureConnections' AS URL
                                     , 'No encrypted communication connections to ['
                                       + @@SERVERNAME
                                       + '] detected. Communication connections to SQL Server should be protected with SSL or TLS.'
                                FROM   sys.dm_exec_connections
                                WHERE  encrypt_option = 'FALSE';

                END;

            /*3 - V40932 - MaxFileSize*/
            IF @CheckType IN ( 1 )
                BEGIN

                    EXEC sp_MSforeachdb 'use [?];
					IF DB_ID(''?'') > 4
					INSERT  INTO #WOXResults
						( CheckID,
						  Databasename,
						  Severity,
						  CheckGroup,
						  Finding,
						  URL,
						  Details
						)
					SELECT  3 AS CheckID,
							DB_Name(),
							1 AS Priority,
							''INSTANCE'' AS FindingGroup,
							''V-40932''  AS Finding,
							''http://WaterOxConsulting.com/Comliance/MaxFileSize'' AS URL,
							''The transaction log file for ['' + DB_NAME() + ''] is configured with a maximum filesize which may result in the SQL Server stopping if the log fills up and can not grow.''
					FROM [?].sys.database_files
					WHERE type_desc = ''LOG''
					AND state = 0
					and max_size not in (-1, 268435456)';

                END;

            /*4 - V40932 - PercentGrowth*/
            IF @CheckType IN ( 1 )
                BEGIN

                    EXEC sp_MSforeachdb 'use [?];
						IF DB_ID(''?'') > 4
					INSERT  INTO #WOXResults
						( CheckID ,
						  Databasename ,
						  Severity ,
						  CheckGroup ,
						  Finding ,
						  URL ,
						  Details
						)
					SELECT  distinct 
							4 AS CheckID ,
							DB_Name() ,
							1 AS Priority ,
							''INSTANCE'' AS FindingGroup ,
							''V-40932''  AS Finding ,
							''http://WaterOxConsulting.com/archives/Compliance/PercentGrowth'' AS URL ,
							''The transaction log file for ['' + DB_NAME() + ''] is configured to grow by ''+convert(varchar(5),growth)+''%. On a highly active system this can cause exponential growth and possible fill up the log file resulting in SQL Server being unable to run.''
							FROM [?].sys.database_files
							WHERE type_desc = ''LOG''
							AND state = 0
							and is_percent_growth = 1
							';
                END;
            /*5 - V41411 - DataEncryption*/
            IF @CheckType IN ( 1 )
                BEGIN
                    EXEC sp_MSforeachdb 'use [?];
		IF db_id(''?'') > 4
		if (select isnull(count(1),0) from sys.symmetric_keys) = 0 
		begin
			INSERT  INTO #WOXResults
			(CheckID,DatabaseName,Severity,CheckGroup,Finding,URL,Details) 
			SELECT  
			5 AS CheckID, DB_Name(),1 AS Priority,''Database'' AS FindingGroup,''V-41412''AS Finding,
			''http://WaterOxConsulting.com/archives/Compliance/Encryption'' AS URL,
			''No Symmetric keys were found for encryption. Ensure sensitive data is encrypted in the application or database.''
		end else
		begin
			INSERT  INTO #WOXResults
			(CheckID,DatabaseName,Severity,CheckGroup,Finding,URL,Details) 
		SELECT DISTINCT
			5 AS CheckID,DB_Name(),1 AS Priority,''Database'' AS FindingGroup,''V-41412''AS Finding,
			''http://WaterOxConsulting.com/archives/Compliance/Encryption'' AS URL,
			case when (SELECT count(1) FROM INFORMATION_SCHEMA.COLUMNS WHERE DATA_TYPE in(''varbinary'',''NVarbinary'')) = 0 then ''There are symmetric keys for encryption, but no data is stored in binary format. Verify sensitive data is encrypted & stored appropriately.'' 
			else ''There are symmetric keys and binary columns that may store encrypted data. Ensure sensitive data in the database is encrypted & stored appropriately.'' END
				from sys.symmetric_keys
			INSERT  INTO #WOXResults
			(CheckID,DatabaseName,Severity,CheckGroup,Finding,URL,Details) 
		SELECT DISTINCT
			5 AS CheckID,DB_Name(),1 AS Priority,''Database'' AS FindingGroup,''V-41412''AS Finding,
			''http://WaterOxConsulting.com/archives/Compliance/Encryption'' AS URL,
			case when (SELECT count(1) FROM INFORMATION_SCHEMA.COLUMNS WHERE DATA_TYPE in(''varbinary'',''NVarbinary'')) = 0 then ''There are symmetric keys for encryption, but no data is stored in binary format. Verify sensitive data is encrypted & stored appropriately.'' 
			else ''There are symmetric keys and binary columns that may store encrypted data. Ensure sensitive data in the database is encrypted & stored appropriately.'' END
				from sys.symmetric_keys
		end'        ;
                END;
            /*5 - V41411 - DataEncryption(2)*/
            IF @CheckType IN ( 1 )
                BEGIN
                    EXEC sp_MSforeachdb 'use [?];
					IF db_id(''?'') > 4
					if (select isnull(count(1),0) from sys.asymmetric_keys) = 0 
					begin
						INSERT  INTO #WOXResults
						(CheckID,DatabaseName,Severity,CheckGroup,Finding,URL,Details) 
						SELECT  
						5 AS CheckID ,
						DB_Name() ,
						1 AS Priority ,
						''Database'' AS FindingGroup ,
						''V-41412''AS Finding ,
						''http://WaterOxConsulting.com/archives/Compliance/Algorithms'' AS URL ,
						''No asymmetric keys were found for encryption. Ensure sensitive data is encrypted in the application or database.''
					end else
					begin
					INSERT  INTO #WOXResults
						(CheckID,DatabaseName,Severity,CheckGroup,Finding,URL,Details) 
					SELECT DISTINCT
						5 AS CheckID,
						DB_Name(),
						1 AS Priority,
						''Database'' AS FindingGroup,
						''V-41412''AS Finding,
						''http://WaterOxConsulting.com/archives/Compliance/Algorithms'' AS URL,
						case when (SELECT count(1) FROM INFORMATION_SCHEMA.COLUMNS WHERE DATA_TYPE in(''varbinary'',''NVarbinary'')) = 0 then ''There are asymmetric keys for encryption, but no data is stored in binary format. Verify sensitive data is encrypted & stored in the database appropriately.'' 
						else '' There are asymmetric keys and binary data type columns that may be storing encrypted data. Ensure sensitive data in the database is encrypted & stored appropriately.'' END
							from sys.asymmetric_keys
					end';
                END;


            /*6 - V40941 - DataTools*/
            IF @CheckType IN ( 1 )
               AND @EnablePS = 1
                BEGIN

                    INSERT INTO #net_data
                    EXEC master..xp_cmdshell 'powershell.exe -noprofile -command "Get-ItemProperty HKLM:\Software\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\* | Where-Object DisplayName -like "*SQL*Server*Data*Tools*" | SELECT-object DisplayName"';
                    SELECT @cdt = COUNT(*)
                    FROM   #net_data
                    WHERE  descriptions LIKE '%SQL Server Data Tools%';
                    IF @cdt > 0
                        INSERT INTO #WOXResults ( CheckID
                                                , DatabaseName
                                                , Severity
                                                , CheckGroup
                                                , Finding
                                                , URL
                                                , Details )
                                    SELECT DISTINCT 6 AS CheckID
                                         , @@SERVERNAME
                                         , 1 AS Priority
                                         , 'INSTANCE' AS FindingGroup
                                         , 'V-40941' AS Finding
                                         , 'http://WaterOxConsulting.com/archives/Compliance/DataTools' AS URL
                                         , 'SQL Server Data Tools is installed on this server. If this is a documented requirement for this server this is not a finding, otherwise uninstall SSDT.'
                                    FROM   sys.dm_exec_connections
                                    WHERE  encrypt_option = 'FALSE';
                    TRUNCATE TABLE #net_data;
                END;

            /*7 -  V-40945 - STIG compliance requires the latest service patch be applied, this check is for service pack version numbers.*/
            INSERT #Versions ( Ver )
            VALUES ( '10.50.6000.34' )  -- SQL 2008R2 SP3
                 , ( '10.0.6000.29' )   -- SQL 2008 SP4
                 , ( '11.0.7001.0' )    -- SQL 2012 SP4
                 , ( '12.0.5000.0' )    --- SQL 2014 SP2
                 , ( '13.0.4001.0' )    --- SQL 2016
                 , ( '14.0.1000.169' ); --- SQL 2017


            IF @CheckType IN ( 1 )
                BEGIN

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 7 AS CheckID
                                     , @@SERVERNAME
                                     , 1 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40945' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/ServicePack' AS URL
                                     , 'Your patch level is: '
                                       + @ProductVersion
                                       + '. Double check that this is the latest patch you are able to download. Current best practices are to keep up with Cumulative Updates (CU) rather than waiting for full Service Packs.'
                                WHERE  @ProductVersion NOT IN (   SELECT Ver
                                                                  FROM   #Versions );
                END;

            /*8 - V41399 - Check for Auto-run procedures*/

            IF @CheckType IN ( 1 )
                BEGIN

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 8 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41399' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/AutoRun' AS URL
                                     , 'Auto-run stored procedures were found on the server. Verify that the procedures are properly documented as authorized.'
                                FROM   master.sys.procedures
                                WHERE  is_auto_executed = 1;
                END;

            /*9 - V41399 - Check job history*/

            IF @CheckType IN ( 1 )
                BEGIN

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 9 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41399' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/JobHistory' AS URL
                                     , 'Review the SQL Job Job History for any unauthorized jobs having been run.';

                END;
            /*10 - V41395 - Dev accounts with excess access
- Added logic to catch orphaned accounts*/

            IF @CheckType IN ( 1 )
                BEGIN
                    EXEC sp_MSforeachdb 'use [?];
					IF DB_ID(''?'') > 4
				INSERT  #DBUs ( DBUserName)
						SELECT  [DatabaseUserName] = princ.[name]
						FROM    sys.database_principals princ
						WHERE   princ.[type] IN (''S'',''U'',''C'')
						AND princ.authentication_type <> 0 and princ.name <> ''dbo'';
				DECLARE @IntFlag INT = 1 , @intMaxFlag INT = ( SELECT MAX(DBUID) FROM #DBUs), @DBUserName VARCHAR(125), @count tinyint
				WHILE (@IntFlag <= @intMaxFlag)
					BEGIN TRY
						SET @DBUserName = ( SELECT DBUserName FROM #DBUs WHERE DBUID = @IntFlag);
						EXECUTE AS USER =  @DBUserName;
						SELECT  @DBUserName = @DBUserName,@count = COUNT(permission_name)
						FROM    fn_my_permissions(NULL, ''DATABASE'')
						WHERE   permission_name LIKE ''CONTROL%'' OR permission_name LIKE ''ALTER%''
								OR permission_name LIKE ''TAKE%'' OR permission_name LIKE ''VIEW%''
								OR permission_name LIKE ''DELETE%'' OR permission_name LIKE ''UPDATE%''
								OR permission_name LIKE ''INSERT%'' OR permission_name LIKE ''SELECT%''
								OR permission_name LIKE ''CREATE%'';
						REVERT;
								SET @IntFlag = @IntFlag + 1
					if @count <> 0 BEGIN
			     INSERT  INTO #WOXResults
                            ( CheckID,DatabaseName, Severity,CheckGroup, Finding, URL, Details)
					SELECT 10 AS CheckID, db_name(), 2 AS Priority, ''Instance'' AS FindingGroup,
							''V-41395'' AS Finding,''http://WaterOxConsulting.com/archives/Compliance/ExcessAccess'' AS URL,
							''The account [''+ @DBUSerName + ''] can change or alter database objects or data in the database [?].''
					END	END TRY 
					BEGIN CATCH
					SET @IntFlag = @IntFlag + 1
					INSERT INTO #WOXResults
                            (CheckID,DatabaseName,Severity,CheckGroup,Finding,URL,Details)
					SELECT  10 AS CheckID,db_name(),2 AS Priority,''Instance'' AS FindingGroup,''V-41395'' AS Finding,''http://WaterOxConsulting.com/archives/Compliance/ExcessAccess'' AS URL,
							''[''+ @DBUSerName + ''] may be orphaned.''
					END	CATCH
				Truncate TABLE #DBUs;';
                END;


            /*11 - V41394 - With GRANT */
            IF @CheckType IN ( 1 )
                BEGIN
                    EXEC sp_MSforeachdb 'use [?];
					IF DB_ID(''?'') > 4
				 INSERT  INTO #WOXResults
                            ( CheckID ,
                              DatabaseName ,
                              Severity ,
                              CheckGroup ,
                              Finding ,
                              URL ,
                              Details
						    )
					SELECT  DISTINCT
							11 AS CheckID ,
							@@SERVERNAME ,
							2 AS Priority ,
							''Database'' AS FindingGroup ,
							''V-41394'' AS Finding ,
							''http://WaterOxConsulting.com/archives/Compliance/WithGrant'' AS URL ,
							''The account [''+P.name + ''] can grant permissions to other users in the database [?].''
					FROM sys.database_permissions DBPerm
						INNER JOIN sys.database_principals P ON P.principal_id = DBPerm.grantee_principal_id
					WHERE state_desc = ''GRANT_WITH_GRANT_OPTION'';';

                END;

            /*12 - V41397 - Admin Defined Roles*/
            IF @CheckType IN ( 1 )
                BEGIN
                    EXEC sp_MSforeachdb 'USE [?];
					IF DB_ID(''?'') > 4
									WITH    RM ( member_principal_id, role_principal_id )
										  AS ( SELECT   rm1.member_principal_id ,
														rm1.role_principal_id
											   FROM     sys.database_role_members rm1
											   UNION ALL
											   SELECT   d.member_principal_id ,
														rm.role_principal_id
											   FROM     sys.database_role_members rm
														INNER JOIN RM AS d ON rm.member_principal_id = d.role_principal_id
											 )

								 INSERT  INTO #WOXResults
                            ( CheckID ,
                              DatabaseName ,
                              Severity ,
                              CheckGroup ,
                              Finding ,
                              URL ,
                              Details
						    )
					SELECT  DISTINCT
							12 AS CheckID ,
							@@SERVERNAME ,
							2 AS Priority ,
							''Database'' AS FindingGroup ,
							''V-41397'' AS Finding ,
							''http://WaterOxConsulting.com/archives/Compliance/DBRoles'' AS URL ,
											''[''+mp.name + ''] is a member of the DB Administrative Role [''+rp.name+''] in [''+DB_NAME()+''].'' 
									FROM    RM
											JOIN sys.database_principals rp ON ( RM.role_principal_id = rp.principal_id )
											JOIN sys.database_principals mp ON ( RM.member_principal_id = mp.principal_id )
									WHERE   rp.is_fixed_role = 1
											AND rp.name NOT IN ( ''db_datareader'', ''db_datawriter'',
																 ''db_denydatareader'', ''db_denydatawriter'' )';

                END;

            /*13 - V41396 - Prod and Dev */
            IF @CheckType IN ( 1 )
                BEGIN

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 13 AS CheckID
                                     , @@SERVERNAME
                                     , 1 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41396' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/SharedServer' AS URL
                                     , 'Confirm that this SQL Server Instance does not host both dev and production environments.';

                END;

            /*14 - V41391, V41393, 41392, 41389, 40912*/
            IF @CheckType IN ( 1 )
                BEGIN

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 14 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41391' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/SecurityLabels' AS URL
                                     , 'SQL Server does not have built-in labeling of sensitive data. Review the link for options with 3rd party tools';

                END;
            /*15 - V41422 - ConcurrentSessions*/
            IF @CheckType IN ( 1 )
                BEGIN

                    INSERT #CSES ( Name
                                 , Minimum
                                 , Maximum
                                 , Config_value
                                 , Run_value )
                    EXEC sys.sp_configure N'user connections';

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 15 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41422' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/ConcurrentSessions' AS URL
                                     , 'SQL Server is running with '
                                       + CONVERT(VARCHAR(6), #CSES.config_value)
                                       + CASE WHEN #CSES.config_value in (0, 32767) THEN
                                                  ' (SQL Default)'
                                              ELSE ''
                                         END
                                       + ' concurrent connections per session. This should be reviewed and set to an appropriate value.'
                                FROM   #CSES;

                END;
            /*16 - V41421 - Deployment processes*/
            IF @CheckType IN ( 1 )
                BEGIN

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 16 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41421' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/DeploymentGuides' AS URL
                                     , 'Ensure there is proper documentation of the procedures for transfer of data and code from dev/test to production.';

                END;

            /*17 - V41420 - TDE */
            IF @CheckType IN ( 1 )
                BEGIN
                    IF @@version LIKE '%developer%'
                        BEGIN
                            INSERT INTO #WOXResults ( CheckID
                                                    , DatabaseName
                                                    , Severity
                                                    , CheckGroup
                                                    , Finding
                                                    , URL
                                                    , Details )
                                        SELECT DISTINCT 17 AS CheckID
                                             , DB_NAME()
                                             , 2 AS Priority
                                             , 'Database' AS FindingGroup
                                             , 'V-41420' AS Finding
                                             , 'http://WaterOxConsulting.com/archives/Compliance/TDE' AS URL
                                             , 'This edition of SQL Server is for development purposes only. Ensure it is not supporting production environments.';
                        END;
                    ELSE -- check if STD edition, if so TDE is not an option
                    IF (   @@version NOT LIKE '%Enterprise%'
                           AND @@version NOT LIKE '%Datacenter%'
                           AND @@version NOT LIKE '%Developer%' )
                        BEGIN
                            INSERT INTO #WOXResults ( CheckID
                                                    , DatabaseName
                                                    , Severity
                                                    , CheckGroup
                                                    , Finding
                                                    , URL
                                                    , Details )
                                        SELECT DISTINCT 17 AS CheckID
                                             , DB_NAME()
                                             , 2 AS Priority
                                             , 'Database' AS FindingGroup
                                             , 'V-41420' AS Finding
                                             , 'http://WaterOxConsulting.com/archives/Compliance/TDE' AS URL
                                             , 'This edition of SQL Server does not support Transparent Data Encryption.';
                        END;

                    ELSE
                        BEGIN
                            INSERT INTO #WOXResults ( CheckID
                                                    , DatabaseName
                                                    , Severity
                                                    , CheckGroup
                                                    , Finding
                                                    , URL
                                                    , Details )
                                        SELECT DISTINCT 17 AS CheckID
                                             , DB_NAME()
                                             , 2 AS Priority
                                             , 'Database' AS FindingGroup
                                             , 'V-41420' AS Finding
                                             , 'http://WaterOxConsulting.com/archives/Compliance/TDE' AS URL
                                             , 'Check For Transparent Data Encryption.';
                        END;
                    EXEC sys.sp_MSforeachdb 'use [?]; 
								IF NOT EXISTS
										(SELECT 1
										FROM sys.dm_database_encryption_keys
										where database_id = db_id(''?'')
										)begin
										 INSERT INTO #WOXResults
                                    ( CheckID,
                                      DatabaseName ,
                                      Severity,
                                      CheckGroup ,
                                      Finding,
                                      URL,
                                      Details
						            ) SELECT 17 AS CheckID ,
										DB_NAME() ,
										2 AS Priority,
										''Database'' AS FindingGroup,
										''V-41420'' AS Finding,
										''http://WaterOxConsulting.com/archives/Compliance/TDE'' AS URL,
										''The Database [?] TDE status is: No database encryption key present, no encryption''
										end else
								INSERT  INTO #WOXResults
                                    ( CheckID,DatabaseName,Severity,CheckGroup,Finding,URL,Details) 
									SELECT 
										17 AS CheckID ,
										DB_NAME() ,
										2 AS Priority ,
										''Database'' AS FindingGroup ,
										''V-41420'' AS Finding ,
										''http://WaterOxConsulting.com/archives/Compliance/TDE'' AS URL ,
										''The Database [?] TDE status is: ''+CASE encryption_state
															WHEN 1 THEN ''Unencrypted''
															WHEN 2 THEN ''Encryption in progress''
															WHEN 3 THEN ''Encrypted''
															WHEN 4 THEN ''Key change in progress''
															WHEN 5 THEN ''Decryption in progress''
															WHEN 6 THEN ''Protection change in progress'' END
										FROM sys.dm_database_encryption_keys DEK 
										INNER JOIN sys.databases DB 
											on DB.database_id = DEK.database_id
										WHERE
											DEK.database_id = DB_ID(''?'');';

                END;



            /*18 - V41409 / V-40940 / V-40941/ V-40937 / V40938 / V-40939
- Installed Components*/

            IF @ChkSrvName IS NULL /*Detect default or named instance*/
                BEGIN
                    SET @TrueSrvName = 'MSSQLSERVER';
                    INSERT #Svcs ( Svc
                                 , SvcName )
                           SELECT 'MSSQLServerOLAPService'
                                , 'SQL Server Analysis Services' /*Setting up proper service name*/
                           UNION
                           SELECT 'MSFTESQL'
                                , 'SQL Server Full Text Search'
                           UNION
                           SELECT 'ReportServer'
                                , 'SQL Server Reporting Services'
                           UNION
                           SELECT 'SQLSERVERAGENT'
                                , 'SQL Server Agent'
                           UNION
                           SELECT 'MSSQLSERVER'
                                , 'SQL Server'
                           UNION
                           SELECT 'SQLBrowser'
                                , 'SQL Browser'
                           UNION
                           SELECT 'MSDTC'
                                , 'Distributed Transaction Coordinator'
                           UNION
                           SELECT 'SQLWriter'
                                , 'SQL Server VSS Writer';
                END;
            ELSE
                BEGIN
                    SET @TrueSrvName = CAST(SERVERPROPERTY('INSTANCENAME') AS VARCHAR(128));
                    SET @SQLSrv = '$' + @ChkSrvName;
                    INSERT #Svcs ( Svc
                                 , SvcName )
                           SELECT 'MSOLAP' + @SQLSrv
                                , 'SQL Server Analysis Services'
                           UNION
                           SELECT 'MSFTESQL' + @SQLSrv
                                , 'SQL Server Full Text Search'
                           UNION
                           SELECT 'ReportServer' + @SQLSrv
                                , 'SQL Server Reporting Services'
                           UNION
                           SELECT 'SQLAgent' + @SQLSrv
                                , 'SQL Agent'
                           UNION
                           SELECT 'MSSQL' + @SQLSrv
                                , 'SQL Server'
                           UNION
                           SELECT 'SQLBrowser'
                                , 'SQL Browser'
                           UNION
                           SELECT 'MSDTC'
                                , 'Distributed Transaction Coordinator'
                           UNION
                           SELECT 'SQLWriter'
                                , 'SQL Server VSS Writer';
                END;


            IF CHARINDEX('Server 2008', @@Version) > 0
                INSERT #Svcs ( Svc
                             , SvcName )
                VALUES ( 'MsDtsServer100', 'SQL Server Integration Services' );
            IF CHARINDEX('Server 2012', @@Version) > 0
                INSERT #Svcs ( Svc
                             , SvcName )
                VALUES ( 'MsDtsServer110', 'SQL Server Integration Services' );
            IF CHARINDEX('Server 2014', @@Version) > 0
                INSERT #Svcs ( Svc
                             , SvcName )
                VALUES ( 'MsDtsServer120', 'SQL Server Integration Services' );

            IF CHARINDEX('Server 2008', @@Version) > 0
                INSERT #Svcs ( Svc
                             , SvcName )
                VALUES ( 'MSSQLServerADHelper100'
                       , 'SQL Server Active Directory Helper' );
            IF CHARINDEX('Server 2012', @@Version) > 0
                INSERT #Svcs ( Svc
                             , SvcName )
                VALUES ( 'MSSQLServerADHelper110'
                       , 'SQL Server Active Directory Helper' );
            IF CHARINDEX('Server 2014', @@Version) > 0
                INSERT #Svcs ( Svc
                             , SvcName )
                VALUES ( 'MSSQLServerADHelper120'
                       , 'SQL Server Active Directory Helper' );
            -- loop through all the services to see if they are installed.
            DECLARE @IntFlag INT = 1
                  , @intMaxFlag INT = (   SELECT MAX(SVCID)
                                          FROM   #Svcs )
                  , @SVCName VARCHAR(128);

            WHILE ( @IntFlag <= @intMaxFlag )
                BEGIN
                    SELECT @REGKEY = 'System\CurrentControlSet\Services\'
                                     + Svc
                         , @SVCName = SvcName
                    FROM   #Svcs
                    WHERE  SVCID = @IntFlag;


                    INSERT #RegResult ( ResultValue )
                    EXEC master.sys.xp_regread @rootkey = 'HKEY_LOCAL_MACHINE'
                                             , @key = @REGKEY;

                    IF (   SELECT ResultValue
                           FROM   #RegResult ) = 1
                        BEGIN
                            INSERT INTO #WOXResults ( CheckID
                                                    , DatabaseName
                                                    , Severity
                                                    , CheckGroup
                                                    , Finding
                                                    , URL
                                                    , Details )
                                        SELECT 18 AS CheckID
                                             , @@SERVERNAME
                                             , 2 AS Priority
                                             , 'Database' AS FindingGroup
                                             , 'V-41409' AS Finding
                                             , 'http://WaterOxConsulting.com/archives/Compliance/Components' AS URL
                                             , 'The ' + @SVCName
                                               + ' service is installed on this server. Verify that is is required.';
                            TRUNCATE TABLE #RegResult;
                        END;
                    ELSE
                        BEGIN
                            /* This section is useful if you want to monitor which components are not installed
								just remove the commmented out code to add the "not installed display in the end results.
                            INSERT  INTO #WOXResults
                                    ( CheckID ,
                                      DatabaseName ,
                                      Severity ,
                                      CheckGroup ,
                                      Finding ,
                                      URL ,
                                      Details
						            )
                                    SELECT  18 AS CheckID ,
                                            @@SERVERNAME ,
                                            2 AS Priority ,
                                            'Database' AS FindingGroup ,
                                            'V-41409' AS Finding ,
                                            'http://WaterOxConsulting.com/archives/Compliance/Components' AS URL ,
                                            'The '+ @SVCName + ' service is not installed on this server.'*/
                            TRUNCATE TABLE #RegResult;
                        END;

                    SET @IntFlag = @IntFlag + 1;
                END;

            /*18 - V41909 - DQS*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 18 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41409' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/Components' AS URL
                                     , 'This server acts as a Data Quality Services Server.'
                                FROM   sys.databases
                                WHERE  name IN ( 'DQS_MAIN', 'DQS_PROJECTS'
                                               , 'DQS_STAGING_DATA' );
                END;




            /*19 - V41424 - DataTypes */
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 19 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41424' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/DataTypes' AS URL
                                     , 'Ensure the datatypes in your database that applications may be writing to are consistent, correct and accurate.';


                    /* Check for keys with similar names, but different data types.*/
                    EXEC sys.sp_MSforeachdb 'use [?];
						IF DB_ID(''?'') > 4
							WITH DTCols (COLUMN_NAME, QTY) AS 
							(SELECT  DISTINCT
								C1.COLUMN_NAME,
								COUNT(DISTINCT	C1.DATA_TYPE) AS QTY
							FROM INFORMATION_SCHEMA.COLUMNS C1 
							INNER JOIN INFORMATION_SCHEMA.COLUMNS C2 ON C1.COLUMN_NAME = C2.COLUMN_NAME
							WHERE C1.DATA_TYPE <> C2.DATA_TYPE
							GROUP BY C1.COLUMN_NAME
							HAVING count(DISTINCT C1.DATA_TYPE) > 1)
							INSERT  INTO #WOXResults
								( CheckID,
									DatabaseName,
									Severity,
									CheckGroup,
									Finding,
									URL,
									Details
								) 
							SELECT 19,
							DB_Name(),
							2 AS Priority ,
							''Database'' AS FindingGroup ,
							''V-41424'' AS Finding ,
							''http://WaterOxConsulting.com/archives/Compliance/DataTypes'' AS URL ,
							''There are ''+ convert(varchar(8),COUNT(COLUMN_NAME)) +'' distinct columns with the same name yet different data types in [?]''
							FROM DTCols
							having count(COLUMN_NAME) > 0';
                END;

            /*20 - V41404 - ScheduledJobs*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 20 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41404' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/ScheduledJobs' AS URL
                                     , 'Check all scheduled jobs for any that may change Triggers in your databases.';
                END;
            /*21 - V41406 - ScheduledJobs*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 21 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41406' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/ScheduledJobs' AS URL
                                     , 'Check all scheduled jobs for any that may change Stored Procedures in your databases.';
                END;

            /*22 - V41403 - ScheduledJobs*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 22 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41403' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/ScheduledJobs' AS URL
                                     , 'Check all scheduled jobs for any that may change Functions in your databases.';
                END;
            /*23 - V41407 - ObjectOwners*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 23 AS CheckID
                                     , name
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41407' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/ObjectOwners' AS URL
                                     , SUSER_SNAME(owner_sid)
                                       + ' is the owner and thus permitted to own any object in the '
                                       + state_desc + ' ' + name
                                       + ' database. Ensure documentation is up to date.'
                                FROM   sys.databases
                                WHERE  database_id > 4;


                END;

            /*24 - V41035/V41021/V41027/V41028/V41029/V41031/V41032/V41033/V41042/V41305/V41306/V41307 - TraceFlags*/
            IF @CheckType IN ( 1 )
                BEGIN


                    INSERT #ReqTraces ( EventID )
                    VALUES ( 14 )
                         , ( 15 )
                         , ( 18 )
                         , ( 20 )
                         , ( 102 )
                         , ( 103 )
                         , ( 104 )
                         , ( 105 )
                         , ( 106 )
                         , ( 107 )
                         , ( 108 )
                         , ( 109 )
                         , ( 110 )
                         , ( 111 )
                         , ( 112 )
                         , ( 113 )
                         , ( 115 )
                         , ( 116 )
                         , ( 117 )
                         , ( 118 )
                         , ( 128 )
                         , ( 129 )
                         , ( 130 )
                         , ( 131 )
                         , ( 132 )
                         , ( 133 )
                         , ( 134 )
                         , ( 135 )
                         , ( 152 )
                         , ( 153 )
                         , ( 170 )
                         , ( 171 )
                         , ( 172 )
                         , ( 173 )
                         , ( 175 )
                         , ( 176 )
                         , ( 177 )
                         , ( 178 );
                    -- load CTE with traces
                    INSERT #SQLTraces ( TraceID )
                           SELECT DISTINCT traceid
                           FROM   sys.fn_trace_getinfo(0);
                    INSERT #events
                           SELECT EventID
                           FROM   #ReqTraces;
                    SELECT @TCount = ISNULL(MAX(TID), 0)
                    FROM   #SQLTraces;
                    SET @IntFlag = 1;
                    IF @TCount = 0
                        BEGIN
                            INSERT INTO #WOXResults ( CheckID
                                                    , DatabaseName
                                                    , Severity
                                                    , CheckGroup
                                                    , Finding
                                                    , URL
                                                    , Details )
                                        SELECT 24 AS CheckID
                                             , @@SERVERNAME
                                             , 2 AS Priority
                                             , 'INSTANCE' AS FindingGroup
                                             , 'Various' AS Finding
                                             , 'http://WaterOxConsulting.com/archives/Compliance/TraceFlags' AS URL
                                             , 'No SQL Trace Flags are configured on the server. This feature is deprecated after SQL 2014, use Extended Events instead.';

                        END;
                    ELSE
                        WHILE @IntFlag <= @TCount
                            BEGIN
                                SET @TRACEID = (   SELECT TraceID
                                                   FROM   #SQLTraces
                                                   WHERE  TID = @IntFlag );
                                WITH SQLTRACES ( EventID )
                                AS ( SELECT DISTINCT ( eventid )
                                     FROM   sys.fn_trace_geteventinfo(@TRACEID) )
                                DELETE FROM #events
                                FROM #events AS EV
                                     INNER JOIN SQLTRACES AS ST ON ST.EventID = EV.EventID;

                                SET @IntFlag = @IntFlag + 1;
                            END;
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 24 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'Various' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/TraceFlags' AS URL
                                     , 'Required SQL Trace Flag '
                                       + CONVERT(VARCHAR(5), EventID)
                                       + '  for auditing is missing on the server. (This feature is deprecated after SQL 2014, use Extended Events instead.)'
                                FROM   #events;
                END;

            /*25 - V41419 - SMKBackup*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 25 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-41419' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/SMKBackup' AS URL
                                     , 'Ensure your Service Master Key is backed up.';
                END;
            /*26 - V41412/V40918 - Algorithms*/
            IF @CheckType IN ( 1 )
                BEGIN
                    EXEC sp_MSforeachdb 'use [?];
					IF db_id(''?'') > 4
					if (select isnull(count(1),0) from sys.symmetric_keys) = 0 
					begin
					 INSERT  INTO #WOXResults
										( CheckID ,
										  DatabaseName ,
										  Severity ,
										  CheckGroup ,
										  Finding ,
										  URL ,
										  Details
						
										) SELECT  
						26 AS CheckID ,
						DB_Name() ,
						2 AS Priority ,
						''Database'' AS FindingGroup ,
						''V-41412''AS Finding ,
						''http://WaterOxConsulting.com/archives/Compliance/Algorithms'' AS URL ,
						''No symmetric keys were found for encryption.''
					end else
					begin
					 INSERT  INTO #WOXResults
										( CheckID ,
										  DatabaseName ,
										  Severity ,
										  CheckGroup ,
										  Finding ,
										  URL ,
										  Details
						
										)
					SELECT DISTINCT
						26 AS CheckID,
						DB_Name(),
						2 AS Priority,
						''Database'' AS FindingGroup,
						''V-41412''AS Finding,
						''http://WaterOxConsulting.com/archives/Compliance/Algorithms'' AS URL,
						case when algorithm_desc in (''AES_128'',''AES_192'', ''AES_256'',''Triple DES'') then ''There are symmetric keys using the acceptable algorithm: ''+algorithm_desc else 
						''At least 1 symmetric key is using the unacceptable algorithm: ''+algorithm_desc END
						 from sys.symmetric_keys
					end';
                END;
            /*26 - V41412 - Algorithms*/
            IF @CheckType IN ( 1 )
                BEGIN
                    EXEC sp_MSforeachdb 'use [?];
					IF db_id(''?'') > 4
					if (select isnull(count(1),0) from sys.asymmetric_keys) = 0 
					begin
					 INSERT  INTO #WOXResults
										( CheckID ,
										  DatabaseName ,
										  Severity ,
										  CheckGroup ,
										  Finding ,
										  URL ,
										  Details
						
										) SELECT  
						26 AS CheckID ,
						DB_Name() ,
						2 AS Priority ,
						''Database'' AS FindingGroup ,
						''V-41412''AS Finding ,
						''http://WaterOxConsulting.com/archives/Compliance/Algorithms'' AS URL ,
						''No asymmetric keys were found for encryption.''
					end else
					begin
					 INSERT  INTO #WOXResults
										( CheckID ,
										  DatabaseName ,
										  Severity ,
										  CheckGroup ,
										  Finding ,
										  URL ,
										  Details
						
										)
					SELECT DISTINCT
						26 AS CheckID,
						DB_Name(),
						2 AS Priority,
						''Database'' AS FindingGroup,
						''V-41412''AS Finding,
						''http://WaterOxConsulting.com/archives/Compliance/Algorithms'' AS URL,
						case when algorithm_desc in (''AES_128'',''AES_192'', ''AES_256'',''Triple DES'') then ''There are asymmetric keys using the acceptable algorithm: ''+algorithm_desc else 
						''At least 1 asymmetric key is using the unacceptable algorithm: ''+algorithm_desc END
						 from sys.asymmetric_keys
					end';
                END;
            /*27 - V41413 - DMKEncryption */
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 27 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41413' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/DMKEncryption' AS URL
                                     , 'The Database Master Key is backed up with a password. Ensure the password is a strong one.'
                                FROM   sys.symmetric_keys AS s
                                     , sys.key_encryptions AS k
                                WHERE  s.name = '##MS_DatabaseMasterKey##'
                                       AND s.symmetric_key_id = k.key_id
                                       AND k.crypt_type = 'ESKP';
                END;
            /*28 - V41416 - Keys with Credentials */
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 28 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41416' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/DMKPassword' AS URL
                                     , 'The Database Master Key is stored with credentials. This is a violation of STIG requirements.'
                                FROM   [master].sys.master_key_passwords
                                HAVING COUNT(credential_id) > 0;

                END;

            /*29 - V41417 - DoDCerts*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 29 AS CheckID
                                     , @@servername
                                     , 1 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41417' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/DoDCerts' AS URL
                                     , 'Ensure any certificates used, or to be used, to encrypt keys are DoD PKI Certificates.';

                END;

            /*30 - V41415 - DMKbySMK*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 30 AS CheckID
                                     , @@servername
                                     , 1 AS Priority
                                     , 'Database' AS FindingGroup
                                     , 'V-41415' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/DoDCerts' AS URL
                                     , 'The database master key for [' + name
                                       + '] is encrypted with the service master key. Verify this is acceptable with the database owner.'
                                FROM   [master].sys.databases
                                WHERE  is_master_key_encrypted_by_server = 1
                                       AND owner_sid <> 1
                                       AND state = 0;
                END;
            /*31 - AuditLogins*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT DISTINCT 31 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'Audit' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/AuditLogins' AS URL
                             , 'The server is configured to audit '
                               + CASE WHEN @AuditLevel = 0 THEN
                                          'No login attempts'
                                      WHEN @AuditLevel = 1 THEN
                                          'Successful logins only'
                                      WHEN @AuditLevel = 2 THEN
                                          'Failed logins only'
                                      WHEN @AuditLevel = 3 THEN
                                          'Both failed and successful logins'
                                 END;
            /*32 - ErrorLogs*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT DISTINCT 32 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'ErrorLogs' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/ErrorLogs' AS URL
                             , 'The server is configured to retain '
                               + CONVERT(VARCHAR(2), ISNULL(@ErrorLogs, 6))
                               + CASE WHEN ISNULL(@ErrorLogs, 6) = 6 THEN
                                          ' error log files. This is the minimum / default number of logs. It should be increased.'
                                      ELSE
                                          ' error log files. Verify there are enough logs to properly capture events.'
                                 END;

            /*33 - BUILTINAdmin*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT DISTINCT 33 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'BuiltInAdmin' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/BuiltInAdmin' AS URL
                             , r.name
                               + ' is a member of BUILTIN\ADMINSTRATORS. This account should be removed.'
                        FROM   sys.server_role_members AS m
                               JOIN sys.server_principals AS r ON m.role_principal_id = r.principal_id
                               JOIN sys.server_principals AS u ON m.member_principal_id = u.principal_id
                        WHERE  u.name = 'BUILTIN\Administrators';

            /*34 - V-40942 / 40943  SampleDBs*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT DISTINCT 34 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-40943' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/SampleDBs' AS URL
                             , name
                               + ' is a sample database provided by Microsoft. They should not be in production environments.'
                        FROM   master.sys.databases
                        WHERE  name IN ( 'pubs', 'Northwind' )
                               OR name LIKE 'Adventure%Works%';

            /*35 - V-41037 / V-40936 saAccount*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT DISTINCT 35 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-41036/37' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/saAccount' AS URL
                             , 'The [sa] account '
                               + CASE WHEN l.name = 'sa' THEN ' has not '
                                      ELSE ' has '
                                 END
                               + ' been renamed. The password policy is '
                               + CASE WHEN s.is_policy_checked = 1 THEN 'set'
                                      ELSE 'not set'
                                 END + '. The account is '
                               + CASE WHEN s.is_disabled = 0 THEN 'not '
                                      ELSE ''
                                 END + 'disabled.'
                        FROM   sys.server_principals AS l
                               LEFT OUTER JOIN sys.sql_logins AS s ON s.principal_id = l.principal_id
                        WHERE  l.sid = 0x01;

            /*36 - GuestConnect*/

            EXEC master.sys.sp_MSforeachdb 'use [?];
			if db_id(''?'') > 4
INSERT  INTO #WOXResults
( CheckID,
	Databasename,
	Severity,
	CheckGroup,
	Finding,
	URL,
	Details
)
SELECT  36 AS CheckID,
	@@servername,
	2 AS Priority,
	''INSTANCE'' AS FindingGroup,
	''GuestConnect''  AS Finding,
	''http://WaterOxConsulting.com/archives/Compliance/GuestConnect'' AS URL , 
	''The GUEST account has ''+p.permission_name+'' permission granted on [?]''
FROM [?].sys.database_permissions p JOIN [?].sys.database_principals l
ON p.grantee_principal_id= l.principal_id 
WHERE l.name = ''guest'' AND p.[state] = ''G''';

            /*37 - V-41036 AuthenticationMode/V-59857*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 37 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'Various' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/AuthenticationMode' AS URL
                             , CASE WHEN SERVERPROPERTY(
                                             'IsIntegratedSecurityOnly') <> 1 THEN
                                        'The instance is configured for both Windows and SQL Server Authentication. Verify proper password policies are in place.'
                                    ELSE
                                        'The instance is configured for Windows Authentication only.'
                               END;

            /*38 - TDE Key backups*/
            EXEC sys.sp_MSforeachdb 'use [?];
				  INSERT  INTO #WOXResults
                            ( CheckID ,
                              DatabaseName ,
                              Severity ,
                              CheckGroup ,
                              Finding ,
                              URL ,
                              Details
						    )
                            SELECT  DISTINCT
                                    38 AS CheckID ,
                                    @@SERVERNAME ,
                                    2 AS Priority ,
                                    ''Database'' AS FindingGroup ,
                                    ''Cert Backups'' AS Finding ,
                                    ''http://WaterOxConsulting.com/archives/Compliance/TDE'' AS URL ,
                                     CASE WHEN c.pvt_key_last_backup_date IS NULL THEN ''The certificate ''+ c.name+'' has never been backed up.''
									ELSE ''The certificate ''+c.name+'' has not been backed up with in the last 30 days'' end
							FROM sys.certificates c INNER JOIN sys.dm_database_encryption_keys dek ON c.thumbprint = dek.encryptor_thumbprint
							WHERE pvt_key_last_backup_date IS NULL OR pvt_key_last_backup_date <= DATEADD(dd, -30, GETDATE())';

            /*39 - Direct Access Permissions
V-41207, V-41208, V-41209, V-41246, V-41247, V-41248, V-41250, 
V-41251, V-41252, V-41253, V-41254, V-41255, V-41256, V-41257, 
V-41258, V-41259, V-41260, V-41261, V-41262, V-41263, V-41264, 
V-41265, V-41266, V-41267, V-41268, V-41269, V-41270, V-41271, 
V-41273, V-41274, V-41275, V-41276, V-41277, V-41278, V-41279, 
V-41280, V-41281, V-41283, V-41284, V-41285, V-41286, V-41287, 
V-41288, V-41289, V-41290, V-41291, V-41292, V-41293, V-41294, 
V-41295, V-41296, V-41297, V-41298, V-41299, V-41300, V-41302, 
V-41303, V-55805, V-41206

*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 39 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'Various' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/DirectAccess' AS URL
                             , princ.name COLLATE DATABASE_DEFAULT
                               + ' has direct access to the '
                               + perm.permission_name
                               + ' permission. The account is '
                               + CASE WHEN princ.is_disabled = 0 THEN 'not'
                                      ELSE ''
                                 END + ' disabled.'
                        FROM   sys.server_permissions AS perm
                               INNER JOIN sys.server_principals AS princ ON princ.principal_id = perm.grantee_principal_id
                        WHERE  perm.permission_name IN (
                'Administer bulk operations' ,'Alter any availability group'
              , 'Alter any connection', 'Alter any credential'
              , 'Alter any database', 'Alter any endpoint '
              , 'Alter any event notification ', 'Alter any event session '
              , 'Alter any linked server', 'Alter any login'
              , 'Alter any server audit', 'Alter any server role'
              , 'Alter resources', 'Alter server state ', 'Alter Settings '
              , 'Alter trace', 'Authenticate server ' /*   'Connect SQL',*/
              , 'Control server', 'Create any database '
              , 'Create availability group', 'Create DDL event notification'
              , 'Create endpoint', 'Create server role'
              , 'Create trace event notification', 'External access assembly'
              , 'Shutdown', 'Unsafe Assembly', 'View any database'
              , 'View any definition', 'View server state' )
                               AND princ.name NOT LIKE '##MS%##'
                               AND princ.type_desc <> 'SERVER_ROLE';




            /*40 - Server roles which control access*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT   40 AS CheckID
                               , @@servername
                               , 2 AS Priority
                               , 'INSTANCE' AS FindingGroup
                               , 'Server Role Access' AS Finding
                               , 'http://WaterOxConsulting.com/archives/Compliance/AccessRoles' AS URL
                               , perm.permission_name COLLATE DATABASE_DEFAULT
                                 + ' controls access to the '
                                 + perm.permission_name
                                 + ' permission. The role is '
                                 + CASE WHEN princ.is_disabled = 0 THEN 'not'
                                        ELSE ''
                                   END + ' disabled.'
                        FROM     sys.server_permissions AS perm
                                 INNER JOIN sys.server_principals AS princ ON princ.principal_id = perm.grantee_principal_id
                        WHERE    perm.permission_name IN (
                'Administer bulk operations' ,'Alter any availability group'
              , 'Alter any connection', 'Alter any credential'
              , 'Alter any database', 'Alter any endpoint '
              , 'Alter any event notification ', 'Alter any event session '
              , 'Alter any linked server', 'Alter any login'
              , 'Alter any server audit', 'Alter any server role'
              , 'Alter resources', 'Alter server state ', 'Alter Settings '
              , 'Alter trace', 'Authenticate server ', 'Connect SQL'
              , 'Control server', 'Create any database '
              , 'Create availability group', 'Create DDL event notification'
              , 'Create endpoint', 'Create server role'
              , 'Create trace event notification', 'External access assembly'
              , 'Shutdown', 'Unsafe Assembly', 'View any database'
              , 'View any definition', 'View server state' )
                                 AND princ.type_desc = 'SERVER_ROLE'
                        ORDER BY perm.permission_name
                               , princ.name;


            /*41 - V40909 - resource governor*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 41 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-40909' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/ResourceGovernor' AS URL
                             , 'The Resource Governor is enabled on this instance.'
                        FROM   sys.resource_governor_configuration
                        WHERE  is_enabled = 1;

            /*42 - V40946 - DB files away from OS (different drives)*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 42 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-40946' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/OSFiles' AS URL
                             , 'SQL Server data files are located on the OS (C:) drive. They should be moved to seperate drives.'
                        FROM   sys.master_files
                        WHERE  UPPER(LEFT(physical_name, 1)) = 'C'
                               AND DB_NAME(database_id) NOT IN ( 'master'
                                                               , 'model'
                                                               , 'msdb'
                                                               , 'tempdb' );


            /*43 - V-40935 - xp_cmdshell*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 43 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-40935' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/FilesFolders' AS URL
                             , 'xp_cmdshell is actived for the processessing of this script. '
                               + CASE WHEN @LeaveCmd = 1 THEN
                                          +' It will be left active upon completion due to the @LeaveCmd parameter being set to 1'
                                      ELSE
                                          'It will be deactivated up on the completion of this check script.'
                                 END
                        FROM   sys.configurations
                        WHERE  name = 'xp_cmdshell'
                               AND value = 1;




            /*44 - Password Encryption*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 44 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-40921' AS Finding
                             , 'http://WaterOxConsulting.com/archives/Compliance/PasswordEncryption' AS URL
                             , 'As of SQL 2005, SQL Server transmits only encrypted userIDs and passwords. This does not apply to other application UIDS and passwords, just those native to SQL Server.';


            /*45 - 47 DB level checks */

            /*45 - V-60781 - High*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 45 AS CheckID
                             , D.name AS [Database]
                             , 1 AS Priority
                             , 'DATABASE' AS FindingGroup
                             , 'V-60781' AS Finding
                             , 'Coming soon' AS URL
                             , 'The Database Owner '
                               + SUSER_SNAME(D.owner_sid)
                               + ' is a member of the ' + F.RoleName
                               + ' fixed server role and the Trustworthy setting is '
                               + CASE WHEN D.is_trustworthy_on = 1 THEN 'ON'
                                      ELSE 'OFF'
                                 END
                        FROM   sys.databases AS D
                               INNER JOIN (   SELECT 'sysadmin' AS RoleName
                                              UNION
                                              SELECT 'securityadmin'
                                              UNION
                                              SELECT 'serveradmin'
                                              UNION
                                              SELECT 'setupadmin'
                                              UNION
                                              SELECT 'processadmin'
                                              UNION
                                              SELECT 'diskadmin'
                                              UNION
                                              SELECT 'dbcreator'
                                              UNION
                                              SELECT 'bulkadmin' ) AS F ON 1 = 1
                        WHERE  IS_SRVROLEMEMBER(
                                   F.RoleName, SUSER_SNAME(D.owner_sid)) = 1
                               AND D.name <> 'msdb'
                               AND D.is_trustworthy_on = 1;

            /*46 - V-60671 - High*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                  SELECT 46 AS CheckID
                             , D.name AS [Database]
                             , 1 AS Priority
                             , 'DATABASE' AS FindingGroup
                             , 'V-60671' AS Finding
                             , 'Coming soon' AS URL
                             , 'A non-admin login owns the database with the Trustworthy setting is '
                               + CASE WHEN D.is_trustworthy_on = 1 THEN 'ON'
                                      ELSE 'OFF'
                                 END
                        FROM   sys.databases AS D
                        WHERE  (IS_SRVROLEMEMBER('sysadmin', SUSER_SNAME(D.owner_sid)) <> 1
								   OR
								   IS_SRVROLEMEMBER('securityadmin', SUSER_SNAME(D.owner_sid)) <> 1
								   OR
								   IS_SRVROLEMEMBER('serveradmin', SUSER_SNAME(D.owner_sid)) <> 1
								   OR
								   IS_SRVROLEMEMBER('setupadmin', SUSER_SNAME(D.owner_sid)) <> 1
								   OR
								   IS_SRVROLEMEMBER('processadmin', SUSER_SNAME(D.owner_sid)) <> 1
								   OR
								   IS_SRVROLEMEMBER('diskadmin', SUSER_SNAME(D.owner_sid)) <> 1
								   OR
								   IS_SRVROLEMEMBER('db_creator', SUSER_SNAME(D.owner_sid)) <> 1
								   OR
								   IS_SRVROLEMEMBER('bulkadmin', SUSER_SNAME(D.owner_sid)) <> 1)
                               AND D.name <> 'msdb'
                               AND D.is_trustworthy_on = 1;

            /*47 - V-70627/V-41026 - Med - space warnings*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 47 AS CheckID
                             , @@servername
                             , 3 AS Priority
                             , 'DATABASE' AS FindingGroup
                             , 'V-70627/V-41026' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Ensure systems or processes are in place for alerting on low storage space available for database transaction logs & SQL Audit files.';


            /*47+ Instance level checks
V-40948 - High*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 47 AS CheckID
                             , @@servername
                             , 1 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-40948' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Ensure systems are in place for monitoring the change of any SQL Server installed files.';

            /* 48 - V-72415/72413*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 48 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-72415' AS Finding
                             , 'Coming Soon' AS URL
                             , CASE WHEN SERVERPROPERTY(
                                             'IsIntegratedSecurityOnly') <> 1 THEN
                                        'The instance is configured for both Windows and SQL Server Authentication.'
                                    ELSE
                                        'The instance is configured for Windows Authentication only.'
                               END
                               + ' Verify proper password lifetime policies are in place. (Min. 24 hrs, Max. 60 days, Min 5 changes before re-use) as well as complexity requirements (The requirements for password complexity are: a. minimum of 15 Characters, 1 of each of the following character sets: - Upper-case - Lower-case - Numeric - Special characters (e.g. ~ ! @ # $ % ^ & * ( ) _ + = - '' [ ] / ? > <)]; b. Minimum number of characters changed from previous password: 50% of the minimum password length (that is, 8))';


            /*49 - V-43196*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 49 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-43196' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Verify that the domain accounts used for running the SQL Service are only used for SQL Server and not other platofrms.';

            /*50 - V-40950*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 50 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-43196' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Verify that the domain accounts used for running the SQL Service are only used for SQL Server and not other platofrms.';

            /*51 - V-40951 - powershell eventually to check advanced audit settings and report accordingly. for now do manual checks (bleah!)*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 51 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-40951' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Verify that the windows folder that has SQL Server components have advanced auditing enabled for everyone';

            /*52 - V-41047*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 52 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-41047' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Verify that each service account used for SQL Server is unique on the server.';

            /*53 - V-41046   - xxxxxxxxx*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 53 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-41046' AS Finding
                             , 'Coming Soon' AS URL
                             , '';

            /*54 - V-41038*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 54 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-41038' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Ensure the account used for SQL Server installation is only used for SQL Server install and config.';

            /*55 - V-41039*/

            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 55 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-41039' AS Finding
                             , 'Coming Soon' AS URL
                             , 'DBA accounts and groups should not have excessive host system priveleges for the administration of SQL Server. Verify that excessive access is not granted to the DBA accounts.';


            /*56 - V-41030*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 56 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-41030' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Ensure the account used for SQL Server installation is only used for SQL Server install and config.';


            /*57 - V-40952, V-40953*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 57 AS CheckID
                             , @@servername
                             , 3 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-40952/53' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Check that the directories that SQL Audit information is stored in are configured to protect from unauthorized modification or deletion.';


            /*58 - V-41023*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 58 AS CheckID
                             , @@servername
                             , 3 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-41023' AS Finding
                             , 'Coming Soon' AS URL
                             , 'SQL Server does not support monitoring of the available audit log file space. Exnsure either File Server resource monagaer or a 3rd party tool is used.';

            /*59- V-41034*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 59 AS CheckID
                             , @@servername
                             , 3 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-41034' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Determine if accounts used to access SQL Server are shared amongst individuals. If mulitple users fo ra single account ensure audits or apps capture who the individual using the account are.';


            /*60 - V-70625*/

            IF @CheckType IN ( 1 )
               AND @EnablePS = 1
                BEGIN
                    INSERT INTO #cbs ( CBS )
                    EXEC master..xp_cmdshell 'powershell.exe -noprofile -command "$arrService = Get-Service -Name ""SQL Server Browser""";$CBS = """The SQL Server Browser service is configured for """+$arrService.StartType+""" startup and is """+$arrService.status + """.""";IF ($arrservice.StartType -ne """Disabled""") {$CBS = $CBS + """ It is recommended this service be disabled unless needed.""""};write $CBS';
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 60 AS CheckID
                                     , @@SERVERNAME
                                     , 3 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-70625' AS Finding
                                     , 'Coming Soon' AS URL
                                     , CBS
                                FROM   #cbs;

                    TRUNCATE TABLE #cbs;
                END;



            /*61 - V-40905*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 62 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40906' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Review SQL Server services for configuration to alert when a failure occurs (Recovery tab) if other monitoring services do not exist.';
                END;
            /*62 - V-40906*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 62 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40906' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Review the monitoring of SQL Security errors to ensure any error that could have a negative effect on database security is quickly identified and forwarded to apprpriate personnel';
                END;
            /*63 - V-40908*/
            IF @CheckType IN ( 1 )
                BEGIN
                    IF ( SELECT SERVERPROPERTY('IsHadrEnabled')) = 1
                        BEGIN
                            INSERT INTO #WOXResults ( CheckID
                                                    , DatabaseName
                                                    , Severity
                                                    , CheckGroup
                                                    , Finding
                                                    , URL
                                                    , Details )
                                        SELECT 63 AS CheckID
                                             , @@servername
                                             , 2 AS Priority
                                             , 'INSTANCE' AS FindingGroup
                                             , 'V-40908' AS Finding
                                             , 'Coming Soon' AS URL
                                             , 'Verify the '
                                               + member_type_desc + ' '
                                               + member_name + ' that is '
                                               + member_state_desc
                                               + ' is able to handle the workload from all other nodes in the case of a failover.'
                                        FROM   sys.dm_hadr_cluster_members AS dhcm
                                        WHERE  member_type = 0;

                        END;
                    ELSE
                        BEGIN
                            INSERT INTO #WOXResults ( CheckID
                                                    , DatabaseName
                                                    , Severity
                                                    , CheckGroup
                                                    , Finding
                                                    , URL
                                                    , Details )
                                        SELECT 63 AS CheckID
                                             , @@servername
                                             , 2 AS Priority
                                             , 'INSTANCE' AS FindingGroup
                                             , 'V-40908' AS Finding
                                             , 'Coming Soon' AS URL
                                             , 'SQL AAG is not enabled on this instance';
                        END;
                END;

            /*64 - V-40910*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 64 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-40910' AS Finding
                             , 'Coming Soon' AS URL
                             , 'Security related database objects (audits, procedures, application auditing, etc) should be kept separate from other objects within the databases. Check each DB to ensure this is done.';
            /*65 - V-40911*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT   65 AS CheckID
                                       , @@servername
                                       , 2 AS Priority
                                       , 'INSTANCE' AS FindingGroup
                                       , 'V-40910' AS Finding
                                       , 'Coming Soon' AS URL
                                       , ISNULL(
                                             'The database ' + d.name
                                             + ' encryption status is '
                                             + CASE e.encryption_state
                                                    WHEN 0 THEN
                                                        'No database encryption key present, no encryption'
                                                    WHEN 1 THEN 'Unencrypted'
                                                    WHEN 2 THEN
                                                        'Encryption in progress'
                                                    WHEN 3 THEN 'Encrypted'
                                                    WHEN 4 THEN
                                                        'Key change in progress'
                                                    WHEN 5 THEN
                                                        'Decryption in progress'
                                                    WHEN 6 THEN
                                                        'Protection change in progress'
                                               END + '.'
                                           , 'The database ' + d.name
                                             + ' cannot be checked for encryption. It may be offline.')
                                FROM     sys.dm_database_encryption_keys AS e
                                         RIGHT JOIN sys.databases AS d ON DB_NAME(
                                                                              e.database_id) = d.name
                                WHERE    d.name NOT IN ( 'master', 'model'
                                                       , 'msdb' )
                                ORDER BY d.name;
                END;
            /*66 - V-40913/40914*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 66 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40913/40914' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Check the accounts used to access public data in SQL Server at both the OS and database levels. Public access accounts should not be in Server roles or OS Guest roles which grant more than read access. Public accounts shoud also not have access to system databases.';
                END;

            /*67 - V-40915*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 67 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40915' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'If the SQL Server houses publicly available information the accounts used to access the data should not be listed in any OS group other than the Guest role to ensure only the OS is granted connection access.';
                END;

            /*68 - V-40916*/
            INSERT INTO #WOXResults ( CheckID
                                    , DatabaseName
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 68 AS CheckID
                             , @@servername
                             , 2 AS Priority
                             , 'INSTANCE' AS FindingGroup
                             , 'V-40916' AS Finding
                             , 'Coming Soon' AS URL
                             , 'If the SQL Server houses publicly available information the accounts used to access the data should not have any securables directly assigned to them.';


            /*69 - V-40919*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 69 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40919' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Verify that any non-organizational users are individually identified when they connect to the SQL Server instance.';
                END;
            /*70 - V-40922*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 70 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40922' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'SQL Server does encrypt stored passwords within its system datababses. Passwords stored outside of SQL Server, or in user databases should be protected. Verify that proper measures are taken to prevent the storage of any plaintext passwords.';
                END;

            /*71 - V-40923/40924*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 71 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40923/40924' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Review all SQL Server users and determine if any shared accounts exist. Per this guidelines there should be no shared accounts unless individuals accessing the data can be identified. If any generic shared accounts are found they should be removed and applications modified appropriately to receive each user''s identity.';
                END;

            /*72 - V-40925*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 72 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40925' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Review all SQL Server users and determine if any shared accounts exist. Per this guidelines there shoul dbe no shared accounts. If any shared accounts are found they should be removed and applications modified appropriately to receive each user''s identity.';
                END;

            /*73 - V-40926/40927*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 73 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40926/40927' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Review system/OS backups and ensure that SQL Server software libraries are included and are compliant with company RTO and RPO objectives.';
                END;

            /*74 - V-40928*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 74 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40926/40927' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Verify the authorized access list for backup and restoration procedures for SQl Server application files (not databases).';
                END;
            /*75 - V-40929, 40930*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 75 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40929, 40930' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Review the 010 - Backup History output file from sp_-WOXAudit to ensure backups are occurring as expected to comply with company RTO and RPO objectives';
                END;
            /*76 - V-40933*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 76 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40933' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Uusing the SQL Server Configuration manager verify the configuration for protocols and ports is within approved guidelines.';
                END;

            /*77 - V-40934*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 77 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40934' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Verify that all user defined functions and stored procedure are necessary to support essential application and organization operations. Any unauthorized functions or procedures should be removed.';
                END;
            /*78 - V-41022 - audit failure = shutdown*/
            IF @CheckType IN ( 1 )
                BEGIN
                    IF EXISTS (   SELECT 1
                                  FROM   sys.server_audits AS sa
                                  WHERE  sa.on_failure = 1
                                         AND sa.on_failure_desc = 'SHUTDOWN SERVER INSTANCE' )
                        BEGIN

                            INSERT INTO #WOXResults ( CheckID
                                                    , DatabaseName
                                                    , Severity
                                                    , CheckGroup
                                                    , Finding
                                                    , URL
                                                    , Details )
                                        SELECT 78 AS CheckID
                                             , @@servername
                                             , 2 AS Priority
                                             , 'INSTANCE' AS FindingGroup
                                             , 'V-41022' AS Finding
                                             , 'Coming Soon' AS URL
                                             , 'Traceflags for auditing and shutdown are deprecated after SQL 2014. Extended events should be referenced instead.'
                                               + CASE WHEN sa.name IS NOT NULL THEN
                                                          'The server audit '
                                                          + sa.name
                                                          + ' is configured to shutdown the SQL Server service on failure.'
                                                 END
                                        FROM   sys.server_audits AS sa
                                        WHERE  sa.on_failure = 1
                                               AND sa.on_failure_desc = 'SHUTDOWN SERVER INSTANCE';
                        END;
                    ELSE
                        BEGIN
                            INSERT INTO #WOXResults ( CheckID
                                                    , DatabaseName
                                                    , Severity
                                                    , CheckGroup
                                                    , Finding
                                                    , URL
                                                    , Details )
                                        SELECT 88 AS CheckID
                                             , @@servername
                                             , 2 AS Priority
                                             , 'INSTANCE' AS FindingGroup
                                             , 'V-41022' AS Finding
                                             , 'Coming Soon' AS URL
                                             , 'No Audits are configured on the server to shutdown the SQL Server service when the audit fails. Verify this is ok adn documented.';
                        END;
                END;
            /*79 - V-41024/41025*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 79 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-41024/41025' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'The server audit '''
                                       + sfa.name COLLATE DATABASE_DEFAULT
                                       + ''' writes to '
                                       + sfa.type_desc COLLATE DATABASE_DEFAULT
                                       + '. The Max files size is set to '
                                       + CAST(sfa.max_file_size AS VARCHAR(100))
                                       + ' MB with a maximum number of files set to '
                                       + CAST(sfa.max_files AS VARCHAR(6))
                                       + ' (potential space used = '
                                       + CAST(CAST(sfa.max_files AS BIGINT)
                                              * CAST(sfa.max_file_size AS BIGINT) AS VARCHAR(100))
                                       + ' MB). The files are stored in '
                                       + sfa.log_file_path
                                       + sfa.log_file_name + '.'
                                       + CASE WHEN sfa.reserve_disk_space = 0 THEN
                                                  'These files are not limited in their growth.'
                                              ELSE
                                                  'These files have reserved '
                                                  + CAST(sfa.reserve_disk_space AS VARCHAR(100))
                                                  + ' MB on the drive.'
                                         END
                                       + ' Ensure that auditing will not outgrow the space reserved.'
                                FROM   sys.server_file_audits AS sfa
                                WHERE  type = 'FL';
                END;
            /*80 - V-40944/V-69169*/
            /* For further development later
				--	  Declare @InstanceName VARCHAR(100)
			   --               , @InstanceLocation VARCHAR(100)
			   --               , @InstancePath VARCHAR(100);;


			   --         /* Build the table to store results */
			   --         IF OBJECT_ID('tempdb..#WOXResults') IS NOT NULL
			   --             DROP TABLE #WOXResults;
			   --         CREATE TABLE #WOXResults
			   --             (
			   --                 ID INT IDENTITY(1, 1)
			   --               , CheckID INT
			   --               , DatabaseName NVARCHAR(128)
			   --               , Severity TINYINT
			   --               , CheckGroup VARCHAR(50)
			   --               , Finding VARCHAR(200)
			   --               , URL VARCHAR(200)
			   --               , Details NVARCHAR(4000)
			   --             );


			   --         /* this is where other check temp tables can go */

			   ----         drop TABLE #net_data
			   ----             (
			   ----                 descriptions VARCHAR(256)
			   ----             );
						----drop TABLE #FoldPerm
						----	(
						----		GNAME VARCHAR(125)
						----	  , UName VARCHAR(125)
						----	  , AccessType VARCHAR(255)
						----	  , Rights VARCHAR(255)
						----	);
						

			  
           
					 --  /* verify the value is correct before trying to pass it to PS script*/ 
					 --  SELECT @InstancePath;

			   --         /*step 2 - verify permissions of the path via PS*/
						---- C:\Dropbox\WaterOx\WOXAudit\POSH\V40944.ps1
						--DECLARE @script NVARCHAR(500)
						--SET @script = 'powershell.exe -noprofile -command C:\Scripts\V40944.ps1 '''+ @instancepath+'''';
						-- --INSERT INTO #FoldPerm ( GNAME
						-- --                      , UName
						-- --                      , AccessType
						-- --                      , Rights )
			
			   --                 EXEC master..xp_cmdshell @script
                   
						--	   --SELECT * FROM #FOLDPERM*/
            IF @CheckType IN ( 1 )
                BEGIN
                    SELECT @InstanceName = @@SERVICENAME;
                    EXEC master..xp_regread @rootkey = 'HKEY_LOCAL_MACHINE'
                                          , @key = 'Software\Microsoft\Microsoft SQL Server\Instance Names\SQL'
                                          , @value_name = @InstanceName
                                          , @value = @InstanceLocation OUTPUT;
                    SELECT @InstanceLocation = 'Software\Microsoft\Microsoft SQL Server\'
                                               + @InstanceLocation + '\Setup';

                    EXEC master..xp_regread @rootkey = 'HKEY_LOCAL_MACHINE'
                                          , @key = @InstanceLocation
                                          , @value_name = 'SQLBinRoot'
                                          , @value = @InstancePath OUTPUT;

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 80 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'Various' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Execute the V40944.ps1 script against the folder: '
                                       + @InstancePath
                                       + ' and review the accounts listed for appropriate permissions against the install directory of SQL Server.';
                END;

            /*81 - V-40947*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 81 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40947' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Verify the accounts used for SQL Server installation are properly restrcited and used only for installation/patching of the instance.';
                END;
            /*82 - V-40949*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 82 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40949' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Verify the processes docmented on how SQL Server monitors for security-relevant configuration settings being changed. (third party tools or baseline and compare with SQL Scripts)';
                END;
            /*83 - V-41016, V-41017*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 83 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-41016/41017' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Execute the V40944.ps1 script against the folder: '''
                                       + LEFT(path, ( LEN(path)
                                                      - CHARINDEX(
                                                            '\'
                                                          , REVERSE(path))
                                                      + 1 ))
                                       + ''' and review the accounts listed for appropriate permissions against the Audit logs for SQL Server.'
                                FROM   sys.traces
                                UNION
                                SELECT 83 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-40944' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Execute the V40944.ps1 script against the folder: '''
                                       + log_file_path
                                       + ''' and review the accounts listed for appropraite permissions against the Audit logs for SQL Server.'
                                FROM   sys.server_file_audits;
                END;

            /*84 - V-41041*/
            BEGIN
                INSERT INTO #WOXResults ( CheckID
                                        , DatabaseName
                                        , Severity
                                        , CheckGroup
                                        , Finding
                                        , URL
                                        , Details )
                            SELECT 84 AS CheckID
                                 , @@servername
                                 , 2 AS Priority
                                 , 'INSTANCE' AS FindingGroup
                                 , 'V-41041' AS Finding
                                 , 'Coming Soon' AS URL
                                 , 'Determine the permissions of any OS or domain accounts used to run external procedures (SQLCLR) are only those that are necessary to perform the required functionality.';
            END;


            /*85 - V-41043*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 85 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-41043' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Review the output from the ''012 - SQL SysAdmins.ps1'' results and verify all SysAdmin accounts are valid. Adminsitrators must use a separate admin account whn performing administrative activities and accessing security functions and information.';
                END;
            /*86 - V-41044 - admin perms*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 86 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-41044' AS Finding
                                     , 'Coming Soon' AS URL
                                     , USER_NAME(
                                           dppriper.grantee_principal_id)
                                       + ' has explicite '
                                       + dppriper.state_desc + ' on '
                                       + OBJECT_NAME(dppriper.major_id)
                                       + '. This permission should not be granted directly.'
                                FROM   sys.database_permissions AS dppriper
                                       INNER JOIN sys.database_principals AS dppri ON dppriper.grantee_principal_id = dppri.principal_id
                                       INNER JOIN #AdminPerms ON OBJECT_NAME(
                                                                     dppriper.major_id) = #AdminPerms.Perm
                                WHERE  grantee_principal_id <> 0;




                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 86 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-41044' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Verify that ' + UPPER(SP2.name)
                                       + ' is a valid member of the fixed server role '
                                       + UPPER(SP.name)
                                FROM   sys.server_role_members AS SRM
                                       INNER JOIN sys.server_principals AS SP ON SP.principal_id = SRM.role_principal_id
                                       INNER JOIN sys.server_principals AS SP2 ON SP2.principal_id = SRM.member_principal_id
                                WHERE  SP.is_fixed_role = 1;


                    -- repeat for each DB on the server for its fixed roles too.


                    EXEC sp_MSforeachdb 'use [?];
						INSERT INTO #WOXResults ( CheckID
									, DatabaseName
									, Severity
									, CheckGroup
									, Finding
									, URL
									, Details )
						SELECT 86 AS CheckID
								, ''[?]''
								, 2 AS Priority
								, ''DATABASE'' AS FindingGroup
								, ''V-41044'' AS Finding
								, ''Coming Soon'' AS URL
								, ''Verify that '' + UPPER(DP2.name) + '' is a valid member of the fixed database role '' + UPPER(DP.name)+''''
						FROM   sys.database_role_members AS SRM
							   INNER JOIN sys.database_principals AS DP ON DP.principal_id = SRM.role_principal_id
							   INNER JOIN sys.database_principals AS DP2 ON DP2.principal_id = SRM.member_principal_id
						WHERE  DP.is_fixed_role = 1;';
                END;

            /*87 - V-41045*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 87 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-41045' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Verify that database clients use different sets of credentials to connect dependent on the access required.';
                END;

            /*88 - V-41202/V-41205/V-1304*/
            IF @CheckType IN ( 1 )
                BEGIN
                    DECLARE @admin_Account_name sysname;
                    SET @admin_Account_name = 'NO administrator account found';
                    DECLARE @server_name sysname;
                    SET @server_name = 'NO Server found';

                    SELECT @server_name = name
                    FROM   sys.servers
                    WHERE  server_id = 0;
                    SET @admin_Account_name = @server_name + '\Administrator';

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT   88 AS CheckID
                                       , @@servername
                                       , 2 AS Priority
                                       , 'INSTANCE' AS FindingGroup
                                       , 'Various' AS Finding
                                       , 'Coming Soon' AS URL
                                       , 'The user '
                                         + UPPER(pr.name) COLLATE DATABASE_DEFAULT
                                         + N' has ' + UPPER(pe.state_desc)
                                         + N' on ' + UPPER(pe.permission_name)
                                         + N'. Verify this permissions is valid.'
                                FROM     sys.server_permissions AS pe
                                         JOIN sys.server_principals AS pr ON pe.grantee_principal_id = pr.principal_id
                                         JOIN sys.server_principals AS ps ON pe.grantor_principal_id = ps.principal_id
                                         LEFT JOIN sys.server_principals AS us ON us.principal_id = pe.major_id
                                WHERE    pr.type IN ( 'K', 'S', 'U', 'R' )
                                         AND pe.grantee_principal_id > 10
                                         AND NOT pr.name IN (
                        '##MS_PolicyEventProcessingLogin##'
                      , '##MS_PolicyTsqlExecutionLogin##'
                      , 'NT AUTHORITY\NETWORK SERVICE', 'NT AUTHORITY\SYSTEM'
                      , 'NT SERVICE\MSSQLSERVER', 'NT SERVICE\SQLSERVERAGENT'
                      , 'NT SERVICE\SQLWriter', 'NT SERVICE\Winmgmt' )
                                         AND NOT pr.name = @admin_Account_name
                                         AND pe.permission_name <> 'CONNECT SQL'
                                ORDER BY CASE pr.type
                                              WHEN 'K' THEN 1
                                              WHEN 'S' THEN 2
                                              WHEN 'U' THEN 3
                                              ELSE 4
                                         END;
                END;

            /*89 - V-41204*/
            IF @CheckType IN ( 1 )
                BEGIN

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 89 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-41204' AS Finding
                                     , 'http://WaterOxConsulting.com/archives/Compliance/WithGrant' AS URL
                                     , 'The account [' + SP.name
                                       + '] can grant permissions to other users on the instance.'
                                FROM   master.sys.server_permissions
                                       INNER JOIN master.sys.server_principals AS SP ON grantee_principal_id = SP.principal_id
                                WHERE  state_desc = 'GRANT_WITH_GRANT_OPTION';
                END;


            /*90 - V-41311*/
            IF @CheckType IN ( 1 )
                BEGIN

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 89 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-41204' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'The number of concurrent connections by any account should be limited. Check if any Server Level triggers are used to limit the number of concurrent SQL Server sessions each system account can have. If tno triggers are found verify there is another process enforcing this requirement.';

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT DISTINCT 89 AS CheckID
                                     , @@SERVERNAME
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-41204' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'A server level trigger named ' + name
                                       + ' was found on this instance.'
                                FROM   master.sys.server_triggers;
                END;
            /*91 - V-54859*/

            IF @CheckType IN ( 1 )
                BEGIN
                    SELECT @InstanceName = @@SERVICENAME;
                    EXEC master..xp_regread @rootkey = 'HKEY_LOCAL_MACHINE'
                                          , @key = 'Software\Microsoft\Microsoft SQL Server\Instance Names\SQL'
                                          , @value_name = @InstanceName
                                          , @value = @InstanceLocation OUTPUT;
                    SELECT @InstanceLocation = 'Software\Microsoft\Microsoft SQL Server\'
                                               + @InstanceLocation + '\Setup';

                    EXEC master..xp_regread @rootkey = 'HKEY_LOCAL_MACHINE'
                                          , @key = @InstanceLocation
                                          , @value_name = 'SQLDataRoot'
                                          , @value = @InstancePath OUTPUT;

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 91 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-54859' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Execute the V40944.ps1 script against the folder: '
                                       + @InstancePath
                                       + ' and review the accounts listed for appropriate permissions against the data directory of SQL Server.';
                END;
            /*92 - V-54879*/
            IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 92 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-54879' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Execute the V40944.ps1 script against the folder: '
                                       + LEFT(physical_name, ( LEN(physical_name)
                                                               - CHARINDEX(
                                                                     '\'
                                                                   , REVERSE(
                                                                         physical_name))
                                                               + 1 ))
                                       + 'and review the accounts listed for approriate permissions against the data directories of SQL Server.'
                                FROM   sys.master_files
                                WHERE  database_id > 4
                                       AND type = 0;
                END;

            /*93 - V-54881*/
			 IF @CheckType IN ( 1 )
                BEGIN
                    SELECT @InstanceName = @@SERVICENAME;
                    EXEC master..xp_regread @rootkey = 'HKEY_LOCAL_MACHINE'
                                          , @key = 'Software\Microsoft\Microsoft SQL Server\Instance Names\SQL'
                                          , @value_name = @InstanceName
                                          , @value = @InstanceLocation OUTPUT;
                    SELECT @InstanceLocation = 'Software\Microsoft\Microsoft SQL Server\'
                                               + @InstanceLocation + '\MSSQLServer';

                    EXEC master..xp_regread @rootkey = 'HKEY_LOCAL_MACHINE'
                                          , @key = @InstanceLocation
                                          , @value_name = 'BackupDirectory'
                                          , @value = @InstancePath OUTPUT;

                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 93 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-54881' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Execute the V40944.ps1 script against the folder: '
                                       + @InstancePath
                                       + ' and review the accounts listed for appropriate permissions against the data directory of SQL Server.';
                END;

            /* - V-59857*/

			 IF @CheckType IN ( 1 )
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , DatabaseName
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                                SELECT 94 AS CheckID
                                     , @@servername
                                     , 2 AS Priority
                                     , 'INSTANCE' AS FindingGroup
                                     , 'V-59857' AS Finding
                                     , 'Coming Soon' AS URL
                                     , 'Verify that priveleged accounts are not used for non-administrative purposes. If no policy exists for this this is considered a finding and proper doucmentation should be created.'
                                FROM   sys.master_files
                                WHERE  database_id > 4
                                       AND type = 0;
                END;





            /* credits & acknowledgements*/
            IF @CheckType = 1
                BEGIN
                    INSERT INTO #WOXResults ( CheckID
                                            , Severity
                                            , CheckGroup
                                            , Finding
                                            , URL
                                            , Details )
                    VALUES ( -1, 0, 'DoD STIG SQL Server Compliance Check'
                           , 'From WaterOx Consulting'
                           , 'https://WaterOxConsulting.com/Compliance'
                           , 'STIG Checks based on DoD STIGS as of: 06/23/2015* ' );
                END;
            INSERT INTO #WOXResults ( CheckID
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
            VALUES ( -2, 254, 'Thanks!', 'From WaterOx Consulting'
                   , 'https://WaterOxConsulting.com/Compliance'
                   , 'Thanks from the WaterOx Consulting team.  We hope you found this tool useful, and if you need help with your SQL Server complaince, email us at compliance@WaterOxConsulting.com' );
            INSERT INTO #WOXResults ( CheckID
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
            VALUES ( -2, 255, 'More Thanks!', 'From WaterOx Consulting'
                   , 'https://WaterOxConsulting.com/Compliance'
                   , 'Thanks also go out to our Beta testers (You know who you are!) for helping review the script before its initial release.' );

            INSERT INTO #WOXResults ( CheckID
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
            VALUES ( -1, 0
                   , 'sp_WOXComplaint (TM) v' + CAST(@Version AS VARCHAR(20))
                     + ' as of '
                     + CAST(CONVERT(DATE, @VersionDate, 102) AS VARCHAR(100))
                   , 'From WaterOx Consulting'
                   , 'https://WaterOxConsulting.com/Complaince', '' );

            INSERT INTO #WOXResults ( CheckID
                                    , Severity
                                    , CheckGroup
                                    , Finding
                                    , URL
                                    , Details )
                        SELECT 999
                             , 253
                             , 'Rundate'
                             , GETDATE()
                             , 'https://WaterOxConsulting.com/Compliance'
                             , '';
            /* disable excess permissions granted at beginning of script for SQLCmd*/
            EXEC sp_configure 'show advanced options'
                            , 1;
            RECONFIGURE;
            EXEC sp_configure 'xp_cmdshell'
                            , @LeaveCmd;
            RECONFIGURE;
            EXEC sp_configure 'show advanced options'
                            , 0;
            RECONFIGURE;


            IF @OutputType <> 'NONE'
                BEGIN
                    SELECT   CheckID
                           , DatabaseName
                           , Severity
                           , CheckGroup
                           , Finding
                           , URL
                           , Details
                           , ID
                    FROM     #WOXResults
                    WHERE    Details IS NOT NULL
                    ORDER BY Severity
                           , CheckGroup
                           , Finding
                           , Details;
                END;

        END;
GO

/*execute the proc once created, then drop the proc*/
EXEC dbo.sp_WOXCompliant_Auto @checktype = 1;
GO

DROP PROCEDURE dbo.sp_WOXCompliant_Auto;
GO
